import torch
import torch.nn as nn
import torch.nn.functional as F

import numpy as np

import torchvision
import torchvision.datasets as datasets

import os

##################################################################################
##################################################################################
##################################################################################

dataset_root = f'{os.getenv("HOME")}/torch_dataset'

def fmnist(train):
	fnist=datasets.FashionMNIST(dataset_root, download=True, train=train)
	fnist_x = (fnist.data/255).view(-1,1,28,28)
	fnist_y = F.one_hot(fnist.targets).float()
	return (fnist_x,fnist_y)

def mnist(train):
	mnist=datasets.MNIST(dataset_root, download=True, train=train)
	mnist_x = (mnist.data/255).view(-1,1,28,28)
	mnist_y = F.one_hot(mnist.targets).float()
	return (mnist_x,mnist_y)

##################################################################################
##################################################################################
##################################################################################

def mnist_fc(layers,σ,linear_gen=lambda λ1,λ2: nn.Linear(λ1,λ2),dropout=0):
	layers = [int(np.round(l)) for l in layers]
	disable_drp = 1 if dropout==0 else 0
	return [nn.Flatten()] + ([
		linear_gen(λ1,λ2)  if index==0 else
		σ                  if index==1 else
		nn.BatchNorm1d(λ2) if index==2 else
		nn.Dropout(dropout)
		for λ1,λ2 in zip(layers[:-1],layers[1:])
		for index in range(4 - disable_drp)
	][:-3 + disable_drp]) # remove last GELU, BatchNorm and Dropout

##################################################################################
##################################################################################
##################################################################################

from sklearn.metrics import confusion_matrix,precision_score,recall_score,accuracy_score,f1_score

from rendering import matrix_save_image, matrix_rgb_save

# confusion matrix
def mnist_perf(model,device,set_name,set_x,set_y,batch_size,rgb=[1,0,0],extra=True):
	classes = [0,1,2,3,4,5,6,7,8,9]
	y_re = torch.tensor([])
	y_nn = torch.tensor([])
	for batch in range(0,set_x.shape[0],batch_size):
		x = set_x[batch:batch+batch_size].to(device)
		y = set_y[batch:batch+batch_size]
		y_re = torch.hstack((y_re,torch.argmax(      y , dim=1)))
		y_nn = torch.hstack((y_nn,torch.argmax(model(x), dim=1).cpu()))
	# 
	cm        = confusion_matrix(y_re,y_nn,labels=classes,)
	# 
	# FP = np.sum(cm.sum(axis=0) - np.diag(cm) )
	# FN = np.sum(cm.sum(axis=1) - np.diag(cm) )
	# TP = np.sum(np.diag(cm)                  )
	# TN = np.sum(cm.sum() - (FP + FN + TP)    )
	# precision = TP/(TP+FP)
	# recall    = TP/(TP+FN)
	# accuracy  = (TP+TN)/(TP+TN+FP+FN)
	# F1        = 2*(precision*recall)/(precision+recall)
	# 
	precision = None
	recall = None
	accuracy = None
	F1 = None
	if extra: precision = precision_score (y_re,y_nn,labels=classes, average='macro', zero_division=0) # zero_division='warn')
	if extra: recall    = recall_score    (y_re,y_nn,labels=classes, average='macro', zero_division=0) # zero_division='warn')
	accuracy            = accuracy_score  (y_re,y_nn)
	if extra: F1        = f1_score        (y_re,y_nn,labels=classes, average='macro', zero_division=0) # zero_division='warn')
	# print(FP+FN+TP+TN)
	# print(f"{precision} {recall} {accuracy} {F1}")
	# print(f"precision={precision}")
	# print(f"recall={recall}")
	# print(f"accuracy={accuracy}")
	# print(f"F1={F1}")
	cm2 = np.nan_to_num(cm  /cm.sum(axis=0)) # normalize with respect to true labels
	# cm[ground_truth][model_found]
	os.makedirs(f"out/",exist_ok=True)
	# matrix_save_image(cm2, f'out/{model.identifier}-confusion-{set_name}.png')
	matrix_rgb_save(cm2,rgb, f'out/{model.identifier}-confusion-{set_name}.png')
	return cm,accuracy,precision,recall,F1

